<?php

class Performers_View_Model {
	const CACHE_KEY        = 'eb_artist_data_%d';
	const CACHE_TTL_IN_MIN = 30;

	/**
	 * @var array
	 */
	protected $data;

	/**
	 * @var object
	 */
	protected $event;

	/**
	 * @var array
	 */
	protected $link_order = [
		'sitepage',
		'facebook',
		'twitter',
		'instagram',
		'bandcamp',
		'spotify',
		'youtube',
		'wikipedia',
		'applemusic',
	];

	protected $black_listed = [
		'musicbrainz',
		'songkick',
		'myspace',
	];

	/**
	 * @var Performer_Options
	 */
	protected $options;

	public function __construct( $event, Performer_Options $options = null ) {
		$this->event   = $event;
		$this->options = $options;
	}


	public function get_data() {
		$artists = [];
		foreach ( $this->event->top_performances as $performer ) {

			if ( ! $this->is_performer_valid( $performer ) ) {
				continue;
			}
			$artists[] = [
				'title'       => $performer->display_name,
				'bio'         => $performer->artist_extended_data->biography,
				'image'       => count( $performer->artist_extended_data->images ) ?
					$this->get_artist_image(
						$performer->artist_extended_data->images[ 0 ],
						$performer->display_name,
						empty( $performer->artist_extended_data->biography ) ? 'bio-image-no-float' : 'bio-image-right' 
					)
					: '',
				'links'       => $this->get_links( $performer->artist_extended_data->social_media_links ),
				'artist_type' => $performer->performer_role,
			];
		}

		return $artists;
	}

	/**
	 * @param $image
	 * @param $artist_name
	 * @param $class
	 *
	 * @return string
	 */
	public function get_artist_image( $image, $artist_name, $class ) {
		$image_path = eb_imageUrlBuild( $image->url );

		return eb_imageTag( $image_path, '', '', $artist_name, '', $this->options->getBioimagesize(), $class );
	}

	public function is_performer_valid( $performer ) {
		if ( ! empty( $performer->artist_extended_data->biography ) ) {
			return true;
		}
		if ( ! empty( $performer->artist_extended_data->social_media_links ) ) {
			return true;
		}

		return false;
	}

	public function get_icon_wrapper_classes() {
		return $this->display_links_in_dialog() ? 'eb-artist_links_hidden' : '';
	}

	/**
	 * @return string
	 */
	public function get_icon_list_classes() {
		$classes = [];
		if ( $this->display_inline_links() ) {
			$classes[] = 'inline-links';
		}
		if ( $this->display_inline_icons() ) {
			$classes[] = 'inline-icons';
		}

		return implode( ' ', $classes );
	}

	/**
	 * @return bool
	 */
	public function display_links_in_dialog() {
		return ( ! $this->display_inline_icons() && ! $this->display_inline_links() );
	}

	/**
	 * @return bool
	 */
	public function display_inline_links() {
		return in_array( 'inline-links', $this->options->getInclude(), true );
	}

	/**
	 * @return bool
	 */
	public function display_inline_icons() {
		return in_array( 'inline-icons', $this->options->getInclude(), true );
	}

	/**
	 * A way to map a css class to the API link type. Currently it's pretty simple
	 * and only includes one class, but it could expand in the future.
	 *
	 * @param $link_type
	 *
	 * @return mixed
	 */
	public function social_link_class_map( $link_type ) {
		$classes = [
			'sitepage' => 'web',
		];

		return isset( $classes[ $link_type ] ) ? $classes[ $link_type ] : $link_type;
	}

	/**
	 * A way to map a css class to the API link type. Currently it's pretty simple
	 * and only includes one class, but it could expand in the future.
	 *
	 * @param $link_type
	 *
	 * @return mixed
	 */
	public function social_link_title_map( $link_type ) {
		$classes = [
			'sitepage' => __( 'Website', 'eventbrite' ),
		];

		return isset( $classes[ $link_type ] ) ? $classes[ $link_type ] : __( ucfirst( $link_type ), 'eventbrite' );
	}

	/**
	 * @return array
	 */
	public function get_performer_images() {
		$images = [];

		foreach ( $this->event->top_performances as $performer ) {
			if ( !isset($performer->artist_extended_dat) || empty( $performer->artist_extended_data->images ) ) {
				continue;
			}
			$images[] = [
				'url'   => $performer->artist_extended_data->images[ 0 ]->url,
				'title' => $performer->display_name,
			];
		}
		if ( empty( $images ) ) {
			$images[] = [
				'url'   => $this->event->logo ? $this->event->logo->url : '',
				'title' => $this->event->name->text,
			];
		}

		return $images;
	}

	public function get_container_class( $type ) {
		return $type === 'supporter'
			? 'artist-box-support'
			: 'artist-box-headliner';
	}

	/**
	 * @param array $links
	 *
	 * @return array
	 */
	private function get_links( $links ) {
		$ordered_links = [];
		foreach ( $this->link_order as $slug ) {
			$key = $this->search_by_link_type( $slug, $links );

			if ( $key !== false ) {
				$ordered_links[] = $links[ $key ];
				unset( $links[ $key ] );
			}
		}

		$links = array_merge( $ordered_links, $links );

		return $this->remove_black_listed( $links );
	}

	/**
	 * @param $type
	 * @param $array
	 *
	 * @return bool|int|string
	 */
	private function search_by_link_type( $type, $array ) {
		foreach ( $array as $key => $value ) {
			if ( $value->link_type === $type ) {
				return $key;
			}
		}

		return false;
	}

	/**
	 * @param array $links
	 *
	 * @return array
	 */
	private function remove_black_listed( $links ) {
		foreach ( $links as $key => $link ) {
			if ( in_array( $link->link_type, $this->black_listed, true ) ) {
				unset( $links[ $key ] );
			}
		}

		return $links;
	}

}
